import datetime
import os.path as osp
import re

import flask

from .parser import parse


app = flask.Flask('logboard')


@app.route('/')
def index():
    root_dir = osp.abspath(app.config['logdir'])
    df, summary_keys, args_keys, log_keys = parse(root_dir)

    summary_keys_ = []
    for key in summary_keys:
        for pattern in app.config['filter']:
            if re.match(pattern, key):
                break
        else:
            summary_keys_.append(key)
    summary_keys = summary_keys_

    if 'log_dir' in flask.request.args and \
            flask.request.args['log_dir'] not in df.log_dir.values:
        request_args = dict(flask.request.args)
        request_args.pop('log_dir')
        return flask.redirect(flask.url_for('index', **request_args))

    return flask.render_template(
        'index.html',
        timestamp=datetime.datetime.now(),
        root_dir=root_dir,
        summary_df=df,
        summary_keys=summary_keys,
        args_keys=args_keys,
        log_keys=log_keys,
    )


@app.route('/logs/<log_dir>/<path:filename>')
def logs(log_dir, filename):
    root_dir = osp.abspath(app.config['logdir'])
    return flask.send_from_directory(osp.join(root_dir, log_dir), filename)



# flake8: noqa

import argparse
import os.path as osp
import re
import sys
import textwrap
import warnings

import tabulate

from ..parser import parse


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        '--logdir',
        '-l',
        default='logs',
        help='logs dir',
    )
    parser.add_argument(
        '--filter',
        '-f',
        nargs='+',
        default=[],
        help='filter keys',
    )
    parser.add_argument('--keys', action='store_true')
    parser.add_argument(
        '--significant-figures',
        '-s',
        type=int,
        default=3,
        help='significant figures',
    )
    parser.add_argument(
        '--index',
        '-i',
        type=int,
        nargs='+',
        help='index filtering',
    )
    parser.add_argument(
        '--column',
        '-c',
        type=int,
        default=30,
        help='maximum column width (0 for infinite)',
    )
    parser.add_argument(
        '--multi-column',
        action='store_true',
        help='prefer multi-column to vertically shorten the table',
    )
    args = parser.parse_args()

    float_format = '{:.%dg}' % args.significant_figures

    root_dir = osp.abspath(args.logdir)
    df, summary_keys, _, log_keys = parse(root_dir, float_format=float_format)

    if args.index is not None:
        df = df.ix[args.index]

    print(' * Log directory: {}'.format(args.logdir))

    summary_keys = ['log_dir'] + summary_keys

    summary_keys_ = []
    for key in summary_keys:
        for pattern in args.filter:
            if re.match(pattern, key):
                break
        else:
            summary_keys_.append(key)
    summary_keys = summary_keys_

    if args.keys:
        for key in summary_keys:
            print(key)
        sys.exit(0)

    headers = ['']
    for key in summary_keys:
        key_short = key.replace('/', '/\n')
        headers.append(key_short)
        if args.multi_column and key in log_keys:
            headers.append('')

    if df.empty:
        df.columns = summary_keys

    rows = []
    for index, df_row in df[summary_keys].iterrows():
        row = [index]
        for key, x in zip(summary_keys, df_row):
            if isinstance(x, tuple):
                assert len(x) == 2
                if args.multi_column:
                    row.extend(x)
                else:
                    x = '\n'.join(str(xi) for xi in x)
                    row.append(x)
            else:
                x = str(x)
                x = textwrap.wrap(x, width=args.column) if args.column else [x]
                x = '\n'.join(x)
                row.append(x)
        rows.append(row)

    table = tabulate.tabulate(
        tabular_data=rows,
        headers=headers,
        tablefmt='fancy_grid',
        stralign='center',
        showindex=False,
        disable_numparse=True,
    )
    print(table)



import datetime
import json
import os
import os.path as osp

import pandas


def parse(root_dir, **kwargs):
    """parse(root_dir, sort=None, q=None)

    Parse log directory to create a log table.

    Parameters
    ----------
    root_dir: str
        Root directory that contains logs.
    sort: str, optional
        Sort logs.
    q: str, optional
        Query of log keys.
    float_format: str, optional.
        Float value formatting (default: '{:.3g}').

    Returns
    -------
    df: pandas.DataFrame
        Log table.
    summary_keys: list of str
        List of keys from summary.
    args_keys: list of str
        List of keys from args.
    log_keys: list of str
        List of keys from log.
    """

    sort = kwargs.get('sort', None)
    q = kwargs.get('q', None)
    float_format = kwargs.get('float_format', '{:.3g}')

    log_dirs = sorted(os.listdir(root_dir))

    # params
    args_keys = set()
    log_keys = set()
    data = []
    for log_dir in sorted(log_dirs):
        args_file = osp.join(root_dir, log_dir, 'args')
        try:
            with open(args_file) as f:
                args = json.load(f)
        except IOError:
            continue
        args_keys = args_keys | set(args.keys())
        datum = args
        datum['log_dir'] = log_dir

        log_file = osp.join(root_dir, log_dir, 'log')
        try:
            with open(log_file) as f:
                log = json.load(f)
        except IOError:
            data.append(datum)
            continue

        log = pandas.DataFrame(data=log)
        datum['epoch'] = log['epoch'].max()
        datum['iteration'] = log['iteration'].max()
        datum['elapsed_time'] = \
            datetime.timedelta(seconds=int(round(log['elapsed_time'].max())))

        now = datetime.datetime.now()
        mtime = datetime.datetime.fromtimestamp(os.path.getmtime(log_file))
        seconds = (now - mtime).total_seconds()
        datum['updated_at'] = datetime.timedelta(seconds=int(round(seconds)))

        for col in log.columns:
            if col in ['epoch', 'iteration', 'elapsed_time']:
                continue

            key = '{} (max)'.format(col)
            log_keys.add(key)
            index = log[col].idxmax()
            datum[key] = (
                log[col][index], (log['epoch'][index], log['iteration'][index])
            )

            key = '{} (min)'.format(col)
            log_keys.add(key)
            index = log[col].idxmin()
            datum[key] = (
                log[col][index], (log['epoch'][index], log['iteration'][index])
            )

        data.append(datum)

    summary_keys = ['epoch', 'iteration', 'elapsed_time', 'updated_at']
    summary_keys += sorted(args_keys)
    summary_keys += sorted(log_keys)

    df = pandas.DataFrame(data=data, columns=['log_dir'] + summary_keys)

    if sort is not None:
        key = sort
        ascending = True
        if key.startswith('-'):
            key = key[1:]
            ascending = False
        df = df.sort_values(by=key, ascending=ascending)

    # format
    for col in df.columns:
        if col.endswith(' (min)') or col.endswith(' (max)'):
            key = col[:-6]
            values = []
            for value in df[col].values:
                if isinstance(value, tuple):
                    values.append((
                        float_format.format(value[0]),
                        value[1],
                    ))
                else:
                    values.append((value, (float('nan'), float('nan'))))
            df[col] = values
        elif df[col].dtype.name == 'timedelta64[ns]':
            values = []
            for value in df[col]:
                try:
                    value = datetime.timedelta(seconds=value.total_seconds())
                    value = str(value)
                except ValueError:
                    pass
                values.append(value)
            df[col] = values
        else:
            df[col] = df[col].astype(str)

    if q is not None:
        try:
            df = df.query(q)
        except Exception:
            pass

    return df, summary_keys, args_keys, log_keys



import argparse
import contextlib
import os
import socket
import sys

from ..views import app


@contextlib.contextmanager
def redirect_to_devnull(stdout=True, stderr=False):
    with open(os.devnull, 'w') as f:
        if stdout:
            sys.stdout = f
        if stderr:
            sys.stderr = f
        yield
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument('--logdir', default='logs', help='logs dir')
    parser.add_argument('--host', default='0.0.0.0', help='host')
    parser.add_argument('--port', default=None, help='port')
    parser.add_argument('--debug', action='store_true', help='debug mode')
    parser.add_argument(
        '--filter', '-f', nargs='+', default=[], help='filter keys'
    )
    args = parser.parse_args()

    app.config['logdir'] = args.logdir
    app.config['filter'] = args.filter

    os.environ['FLASK_ENV'] = 'development'

    print(' * Log directory: {}'.format(args.logdir))

    if args.port:
        app.run(host=args.host, port=args.port, debug=args.debug)
    else:
        for port in range(7006, 9999):
            try:
                with redirect_to_devnull():
                    app.run(host=args.host, port=port, debug=args.debug)
                    break
            except socket.error:
                pass



from __future__ import print_function

import distutils
import shlex
import subprocess
import sys

from setuptools import find_packages
from setuptools import setup


version = '0.5.8'


if sys.argv[1] == 'release':
    if not distutils.spawn.find_executable('twine'):
        print(
            'Please install twine:\n\n\tpip install twine\n',
            file=sys.stderr,
        )
        sys.exit(1)

    commands = [
        'git pull origin master',
        'git tag v{:s}'.format(version),
        'git push origin master --tag',
        'python setup.py sdist',
        'twine upload dist/logboard-{:s}.tar.gz'.format(version),
    ]
    for cmd in commands:
        subprocess.check_call(shlex.split(cmd))
    sys.exit(0)


def get_long_description():
    with open('README.md') as f:
        long_description = f.read()

    try:
        import github2pypi

        return github2pypi.replace_url(
            slug='wkentaro/logboard', content=long_description
        )
    except Exception:
        return long_description


setup(
    name='logboard',
    version=version,
    packages=find_packages(),
    include_package_data=True,
    install_requires=['flask', 'pandas', 'tabulate'],
    author='Kentaro Wada',
    author_email='www.kentaro.wada@gmail.com',
    description='Monitor and Compare Logs on Browser.',
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    url='http://github.com/wkentaro/logboard',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    entry_points={
        'console_scripts': [
            'logboard=logboard.cli.logboard:main',
            'logtable=logboard.cli.logtable:main',
        ],
    },
)
